/****************************************************************************
** Meta object code from reading C++ file 'account-service.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../accounts-qml-module/src/account-service.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'account-service.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14OnlineAccounts14AccountServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto OnlineAccounts::AccountService::qt_create_metaobjectdata<qt_meta_tag_ZN14OnlineAccounts14AccountServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OnlineAccounts::AccountService",
        "objectHandleChanged",
        "",
        "enabledChanged",
        "displayNameChanged",
        "settingsChanged",
        "autoSyncChanged",
        "credentialsChanged",
        "authenticated",
        "QVariantMap",
        "reply",
        "authenticationError",
        "error",
        "onAuthSessionResponse",
        "SignOn::SessionData",
        "sessionData",
        "onAuthSessionError",
        "SignOn::Error",
        "onCredentialsIdChanged",
        "authenticate",
        "method",
        "mechanism",
        "cancelAuthentication",
        "updateServiceEnabled",
        "enabled",
        "updateSettings",
        "settings",
        "objectHandle",
        "serviceEnabled",
        "provider",
        "service",
        "displayName",
        "accountId",
        "authData",
        "autoSync",
        "credentials",
        "ErrorCode",
        "NoError",
        "NoAccountError",
        "UserCanceledError",
        "PermissionDeniedError",
        "NetworkError",
        "SslError",
        "InteractionRequiredError",
        "UiPolicy",
        "DefaultPolicy",
        "RequestPasswordPolicy",
        "NoUserInteractionPolicy",
        "ValidationPolicy"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'objectHandleChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enabledChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'displayNameChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'settingsChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'autoSyncChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'credentialsChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'authenticated'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'authenticationError'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 12 },
        }}),
        // Slot 'onAuthSessionResponse'
        QtMocHelpers::SlotData<void(const SignOn::SessionData &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'onAuthSessionError'
        QtMocHelpers::SlotData<void(const SignOn::Error &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 12 },
        }}),
        // Slot 'onCredentialsIdChanged'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'authenticate'
        QtMocHelpers::MethodData<void(const QVariantMap &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 15 },
        }}),
        // Method 'authenticate'
        QtMocHelpers::MethodData<void()>(19, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'authenticate'
        QtMocHelpers::MethodData<void(const QString &, const QString &, const QVariantMap &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 }, { QMetaType::QString, 21 }, { 0x80000000 | 9, 15 },
        }}),
        // Method 'authenticate'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(19, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 20 }, { QMetaType::QString, 21 },
        }}),
        // Method 'cancelAuthentication'
        QtMocHelpers::MethodData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'updateServiceEnabled'
        QtMocHelpers::MethodData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
        // Method 'updateSettings'
        QtMocHelpers::MethodData<void(const QVariantMap &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'objectHandle'
        QtMocHelpers::PropertyData<QObject*>(27, QMetaType::QObjectStar, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'enabled'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'serviceEnabled'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'provider'
        QtMocHelpers::PropertyData<QVariantMap>(29, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'service'
        QtMocHelpers::PropertyData<QVariantMap>(30, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'displayName'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'accountId'
        QtMocHelpers::PropertyData<uint>(32, QMetaType::UInt, QMC::DefaultPropertyFlags, 0),
        // property 'settings'
        QtMocHelpers::PropertyData<QVariantMap>(26, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'authData'
        QtMocHelpers::PropertyData<QVariantMap>(33, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'autoSync'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'credentials'
        QtMocHelpers::PropertyData<QObject*>(35, QMetaType::QObjectStar, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ErrorCode'
        QtMocHelpers::EnumData<enum ErrorCode>(36, 36, QMC::EnumFlags{}).add({
            {   37, ErrorCode::NoError },
            {   38, ErrorCode::NoAccountError },
            {   39, ErrorCode::UserCanceledError },
            {   40, ErrorCode::PermissionDeniedError },
            {   41, ErrorCode::NetworkError },
            {   42, ErrorCode::SslError },
            {   43, ErrorCode::InteractionRequiredError },
        }),
        // enum 'UiPolicy'
        QtMocHelpers::EnumData<enum UiPolicy>(44, 44, QMC::EnumFlags{}).add({
            {   45, UiPolicy::DefaultPolicy },
            {   46, UiPolicy::RequestPasswordPolicy },
            {   47, UiPolicy::NoUserInteractionPolicy },
            {   48, UiPolicy::ValidationPolicy },
        }),
    };
    return QtMocHelpers::metaObjectData<AccountService, qt_meta_tag_ZN14OnlineAccounts14AccountServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OnlineAccounts::AccountService::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OnlineAccounts14AccountServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OnlineAccounts14AccountServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14OnlineAccounts14AccountServiceE_t>.metaTypes,
    nullptr
} };

void OnlineAccounts::AccountService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountService *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->objectHandleChanged(); break;
        case 1: _t->enabledChanged(); break;
        case 2: _t->displayNameChanged(); break;
        case 3: _t->settingsChanged(); break;
        case 4: _t->autoSyncChanged(); break;
        case 5: _t->credentialsChanged(); break;
        case 6: _t->authenticated((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 7: _t->authenticationError((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 8: _t->onAuthSessionResponse((*reinterpret_cast<std::add_pointer_t<SignOn::SessionData>>(_a[1]))); break;
        case 9: _t->onAuthSessionError((*reinterpret_cast<std::add_pointer_t<SignOn::Error>>(_a[1]))); break;
        case 10: _t->onCredentialsIdChanged(); break;
        case 11: _t->authenticate((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 12: _t->authenticate(); break;
        case 13: _t->authenticate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3]))); break;
        case 14: _t->authenticate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 15: _t->cancelAuthentication(); break;
        case 16: _t->updateServiceEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->updateSettings((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AccountService::*)()>(_a, &AccountService::objectHandleChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountService::*)()>(_a, &AccountService::enabledChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountService::*)()>(_a, &AccountService::displayNameChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountService::*)()>(_a, &AccountService::settingsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountService::*)()>(_a, &AccountService::autoSyncChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountService::*)()>(_a, &AccountService::credentialsChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountService::*)(const QVariantMap & )>(_a, &AccountService::authenticated, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountService::*)(const QVariantMap & )>(_a, &AccountService::authenticationError, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QObject**>(_v) = _t->objectHandle(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->enabled(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->serviceEnabled(); break;
        case 3: *reinterpret_cast<QVariantMap*>(_v) = _t->provider(); break;
        case 4: *reinterpret_cast<QVariantMap*>(_v) = _t->service(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->displayName(); break;
        case 6: *reinterpret_cast<uint*>(_v) = _t->accountId(); break;
        case 7: *reinterpret_cast<QVariantMap*>(_v) = _t->settings(); break;
        case 8: *reinterpret_cast<QVariantMap*>(_v) = _t->authData(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->autoSync(); break;
        case 10: *reinterpret_cast<QObject**>(_v) = _t->credentials(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setObjectHandle(*reinterpret_cast<QObject**>(_v)); break;
        case 9: _t->setAutoSync(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setCredentials(*reinterpret_cast<QObject**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OnlineAccounts::AccountService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OnlineAccounts::AccountService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OnlineAccounts14AccountServiceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int OnlineAccounts::AccountService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void OnlineAccounts::AccountService::objectHandleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OnlineAccounts::AccountService::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OnlineAccounts::AccountService::displayNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OnlineAccounts::AccountService::settingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OnlineAccounts::AccountService::autoSyncChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OnlineAccounts::AccountService::credentialsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OnlineAccounts::AccountService::authenticated(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void OnlineAccounts::AccountService::authenticationError(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
