/****************************************************************************
** Meta object code from reading C++ file 'account.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../accounts-qml-module/src/account.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'account.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14OnlineAccounts7AccountE_t {};
} // unnamed namespace

template <> constexpr inline auto OnlineAccounts::Account::qt_create_metaobjectdata<qt_meta_tag_ZN14OnlineAccounts7AccountE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OnlineAccounts::Account",
        "objectHandleChanged",
        "",
        "accountIdChanged",
        "enabledChanged",
        "displayNameChanged",
        "synced",
        "removed",
        "onRemoved",
        "onIdentityRemoved",
        "updateDisplayName",
        "displayName",
        "updateEnabled",
        "enabled",
        "sync",
        "remove",
        "RemovalOptions",
        "options",
        "objectHandle",
        "provider",
        "QVariantMap",
        "accountId",
        "accountServiceHandle",
        "RemovalOption",
        "RemoveAccountOnly",
        "RemoveCredentials"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'objectHandleChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'accountIdChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enabledChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'displayNameChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'synced'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'removed'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onRemoved'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onIdentityRemoved'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'updateDisplayName'
        QtMocHelpers::MethodData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Method 'updateEnabled'
        QtMocHelpers::MethodData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Method 'sync'
        QtMocHelpers::MethodData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'remove'
        QtMocHelpers::MethodData<void(RemovalOptions)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<void()>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'objectHandle'
        QtMocHelpers::PropertyData<QObject*>(18, QMetaType::QObjectStar, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'enabled'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'provider'
        QtMocHelpers::PropertyData<QVariantMap>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'displayName'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'accountId'
        QtMocHelpers::PropertyData<uint>(21, QMetaType::UInt, QMC::DefaultPropertyFlags, 1),
        // property 'accountServiceHandle'
        QtMocHelpers::PropertyData<QObject*>(22, QMetaType::QObjectStar, QMC::DefaultPropertyFlags, 0),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'RemovalOption'
        QtMocHelpers::EnumData<enum RemovalOption>(23, 23, QMC::EnumFlags{}).add({
            {   24, RemovalOption::RemoveAccountOnly },
            {   25, RemovalOption::RemoveCredentials },
        }),
        // flag 'RemovalOptions'
        QtMocHelpers::EnumData<RemovalOptions>(16, 23, QMC::EnumIsFlag).add({
            {   24, RemovalOption::RemoveAccountOnly },
            {   25, RemovalOption::RemoveCredentials },
        }),
    };
    return QtMocHelpers::metaObjectData<Account, qt_meta_tag_ZN14OnlineAccounts7AccountE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OnlineAccounts::Account::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OnlineAccounts7AccountE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OnlineAccounts7AccountE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14OnlineAccounts7AccountE_t>.metaTypes,
    nullptr
} };

void OnlineAccounts::Account::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Account *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->objectHandleChanged(); break;
        case 1: _t->accountIdChanged(); break;
        case 2: _t->enabledChanged(); break;
        case 3: _t->displayNameChanged(); break;
        case 4: _t->synced(); break;
        case 5: _t->removed(); break;
        case 6: _t->onRemoved(); break;
        case 7: _t->onIdentityRemoved(); break;
        case 8: _t->updateDisplayName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->updateEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->sync(); break;
        case 11: _t->remove((*reinterpret_cast<std::add_pointer_t<RemovalOptions>>(_a[1]))); break;
        case 12: _t->remove(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Account::*)()>(_a, &Account::objectHandleChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Account::*)()>(_a, &Account::accountIdChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Account::*)()>(_a, &Account::enabledChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Account::*)()>(_a, &Account::displayNameChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Account::*)()>(_a, &Account::synced, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Account::*)()>(_a, &Account::removed, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QObject**>(_v) = _t->objectHandle(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->enabled(); break;
        case 2: *reinterpret_cast<QVariantMap*>(_v) = _t->provider(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->displayName(); break;
        case 4: *reinterpret_cast<uint*>(_v) = _t->accountId(); break;
        case 5: *reinterpret_cast<QObject**>(_v) = _t->accountServiceHandle(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setObjectHandle(*reinterpret_cast<QObject**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OnlineAccounts::Account::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OnlineAccounts::Account::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OnlineAccounts7AccountE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OnlineAccounts::Account::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void OnlineAccounts::Account::objectHandleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OnlineAccounts::Account::accountIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OnlineAccounts::Account::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OnlineAccounts::Account::displayNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OnlineAccounts::Account::synced()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OnlineAccounts::Account::removed()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
