/****************************************************************************
** Meta object code from reading C++ file 'account-service-model.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../accounts-qml-module/src/account-service-model.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'account-service-model.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14OnlineAccounts19AccountServiceModelE_t {};
} // unnamed namespace

template <> constexpr inline auto OnlineAccounts::AccountServiceModel::qt_create_metaobjectdata<qt_meta_tag_ZN14OnlineAccounts19AccountServiceModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OnlineAccounts::AccountServiceModel",
        "countChanged",
        "",
        "includeDisabledChanged",
        "accountIdChanged",
        "accountChanged",
        "applicationIdChanged",
        "providerChanged",
        "serviceTypeChanged",
        "serviceChanged",
        "get",
        "QVariant",
        "row",
        "roleName",
        "count",
        "includeDisabled",
        "accountId",
        "account",
        "applicationId",
        "provider",
        "serviceType",
        "service"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'includeDisabledChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'accountIdChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'accountChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'applicationIdChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'providerChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serviceTypeChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serviceChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'get'
        QtMocHelpers::MethodData<QVariant(int, const QString &) const>(10, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::Int, 12 }, { QMetaType::QString, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'count'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags, 0),
        // property 'includeDisabled'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'accountId'
        QtMocHelpers::PropertyData<quint32>(16, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'account'
        QtMocHelpers::PropertyData<QObject*>(17, QMetaType::QObjectStar, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'applicationId'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'provider'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'serviceType'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'service'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AccountServiceModel, qt_meta_tag_ZN14OnlineAccounts19AccountServiceModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OnlineAccounts::AccountServiceModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OnlineAccounts19AccountServiceModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OnlineAccounts19AccountServiceModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14OnlineAccounts19AccountServiceModelE_t>.metaTypes,
    nullptr
} };

void OnlineAccounts::AccountServiceModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountServiceModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->countChanged(); break;
        case 1: _t->includeDisabledChanged(); break;
        case 2: _t->accountIdChanged(); break;
        case 3: _t->accountChanged(); break;
        case 4: _t->applicationIdChanged(); break;
        case 5: _t->providerChanged(); break;
        case 6: _t->serviceTypeChanged(); break;
        case 7: _t->serviceChanged(); break;
        case 8: { QVariant _r = _t->get((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AccountServiceModel::*)()>(_a, &AccountServiceModel::countChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountServiceModel::*)()>(_a, &AccountServiceModel::includeDisabledChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountServiceModel::*)()>(_a, &AccountServiceModel::accountIdChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountServiceModel::*)()>(_a, &AccountServiceModel::accountChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountServiceModel::*)()>(_a, &AccountServiceModel::applicationIdChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountServiceModel::*)()>(_a, &AccountServiceModel::providerChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountServiceModel::*)()>(_a, &AccountServiceModel::serviceTypeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountServiceModel::*)()>(_a, &AccountServiceModel::serviceChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->rowCount(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->includeDisabled(); break;
        case 2: *reinterpret_cast<quint32*>(_v) = _t->accountId(); break;
        case 3: *reinterpret_cast<QObject**>(_v) = _t->account(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->applicationId(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->provider(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->serviceType(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->service(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setIncludeDisabled(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setAccountId(*reinterpret_cast<quint32*>(_v)); break;
        case 3: _t->setAccount(*reinterpret_cast<QObject**>(_v)); break;
        case 4: _t->setApplicationId(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setProvider(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setServiceType(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setService(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OnlineAccounts::AccountServiceModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OnlineAccounts::AccountServiceModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OnlineAccounts19AccountServiceModelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int OnlineAccounts::AccountServiceModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void OnlineAccounts::AccountServiceModel::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OnlineAccounts::AccountServiceModel::includeDisabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OnlineAccounts::AccountServiceModel::accountIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OnlineAccounts::AccountServiceModel::accountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OnlineAccounts::AccountServiceModel::applicationIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OnlineAccounts::AccountServiceModel::providerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OnlineAccounts::AccountServiceModel::serviceTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OnlineAccounts::AccountServiceModel::serviceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
