/****************************************************************************
** Meta object code from reading C++ file 'credentials.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../accounts-qml-module/src/credentials.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'credentials.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14OnlineAccounts11CredentialsE_t {};
} // unnamed namespace

template <> constexpr inline auto OnlineAccounts::Credentials::qt_create_metaobjectdata<qt_meta_tag_ZN14OnlineAccounts11CredentialsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OnlineAccounts::Credentials",
        "credentialsIdChanged",
        "",
        "captionChanged",
        "userNameChanged",
        "secretChanged",
        "storeSecretChanged",
        "aclChanged",
        "methodsChanged",
        "synced",
        "removed",
        "onInfo",
        "SignOn::IdentityInfo",
        "info",
        "onStored",
        "id",
        "sync",
        "remove",
        "credentialsId",
        "caption",
        "userName",
        "secret",
        "storeSecret",
        "acl",
        "methods",
        "QVariantMap"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'credentialsIdChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'captionChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'userNameChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'secretChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'storeSecretChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aclChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'methodsChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'synced'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'removed'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onInfo'
        QtMocHelpers::SlotData<void(const SignOn::IdentityInfo &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'onStored'
        QtMocHelpers::SlotData<void(const quint32)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::UInt, 15 },
        }}),
        // Method 'sync'
        QtMocHelpers::MethodData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'remove'
        QtMocHelpers::MethodData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'credentialsId'
        QtMocHelpers::PropertyData<quint32>(18, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'caption'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'userName'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'secret'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'storeSecret'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'acl'
        QtMocHelpers::PropertyData<QStringList>(23, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'methods'
        QtMocHelpers::PropertyData<QVariantMap>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Credentials, qt_meta_tag_ZN14OnlineAccounts11CredentialsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OnlineAccounts::Credentials::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OnlineAccounts11CredentialsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OnlineAccounts11CredentialsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14OnlineAccounts11CredentialsE_t>.metaTypes,
    nullptr
} };

void OnlineAccounts::Credentials::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Credentials *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->credentialsIdChanged(); break;
        case 1: _t->captionChanged(); break;
        case 2: _t->userNameChanged(); break;
        case 3: _t->secretChanged(); break;
        case 4: _t->storeSecretChanged(); break;
        case 5: _t->aclChanged(); break;
        case 6: _t->methodsChanged(); break;
        case 7: _t->synced(); break;
        case 8: _t->removed(); break;
        case 9: _t->onInfo((*reinterpret_cast<std::add_pointer_t<SignOn::IdentityInfo>>(_a[1]))); break;
        case 10: _t->onStored((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1]))); break;
        case 11: _t->sync(); break;
        case 12: _t->remove(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Credentials::*)()>(_a, &Credentials::credentialsIdChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Credentials::*)()>(_a, &Credentials::captionChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Credentials::*)()>(_a, &Credentials::userNameChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Credentials::*)()>(_a, &Credentials::secretChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Credentials::*)()>(_a, &Credentials::storeSecretChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Credentials::*)()>(_a, &Credentials::aclChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Credentials::*)()>(_a, &Credentials::methodsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Credentials::*)()>(_a, &Credentials::synced, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Credentials::*)()>(_a, &Credentials::removed, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<quint32*>(_v) = _t->credentialsId(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->caption(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->userName(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->secret(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->storeSecret(); break;
        case 5: *reinterpret_cast<QStringList*>(_v) = _t->acl(); break;
        case 6: *reinterpret_cast<QVariantMap*>(_v) = _t->methods(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCredentialsId(*reinterpret_cast<quint32*>(_v)); break;
        case 1: _t->setCaption(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setUserName(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setSecret(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setStoreSecret(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setAcl(*reinterpret_cast<QStringList*>(_v)); break;
        case 6: _t->setMethods(*reinterpret_cast<QVariantMap*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OnlineAccounts::Credentials::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OnlineAccounts::Credentials::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14OnlineAccounts11CredentialsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OnlineAccounts::Credentials::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void OnlineAccounts::Credentials::credentialsIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OnlineAccounts::Credentials::captionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OnlineAccounts::Credentials::userNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OnlineAccounts::Credentials::secretChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OnlineAccounts::Credentials::storeSecretChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OnlineAccounts::Credentials::aclChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OnlineAccounts::Credentials::methodsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OnlineAccounts::Credentials::synced()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void OnlineAccounts::Credentials::removed()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
